/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;


/**
 * This class simply stores some data about the Ant project. It is passed with current data to ServerElement class
 * instance and then to AbstractServer derived classes instances. It is currently used to provide information to the
 * Cactus testing back-end.
 * <p/>
 * This class is immutable, i.e.  thread-safe.
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class AntTaskInfo {

    /**
     * This field represents the current Ant project. It is initialized in constructor.
     */
    private final Project project;

    /**
     * This field represents the current Ant task name. It is initialized in constructor.
     */
    private final String taskName;

    /**
     * This field represents the current Ant task location. It is initialized in constructor.
     */
    private final Location location;

    /**
     * This field represents the current Ant target being executed. It is initialized in constructor and then never
     * changed.
     */
    private final Target target;

    /**
     * Creates an AntTaskInfo instance and initializes its fields with the values of like-named parameters.
     *
     * @param project  the current Ant project
     * @param taskName the current Ant task name
     * @param location the current Ant task location
     * @param target   the current Ant target being executed
     *
     * @throws IllegalArgumentException if any arg is <tt>null</tt> or taskName is an empty (trim'd) String
     */
    public AntTaskInfo(final Project project, final String taskName, final Location location, final Target target) {
        if (project == null) {
            throw new IllegalArgumentException("The parameter named [project] was null.");
        }
        if (taskName == null) {
            throw new IllegalArgumentException("The parameter named [taskName] was null.");
        }
        if (taskName.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [taskName] was an empty String.");
        }
        if (location == null) {
            throw new IllegalArgumentException("The parameter named [location] was null.");
        }
        if (target == null) {
            throw new IllegalArgumentException("The parameter named [target] was null.");
        }
        // argument checking added as of
        // https://software.topcoder.com/forum/c_forum_message.jsp?f=20015788&r=21551214

        this.project = project;
        this.taskName = taskName;
        this.location = location;
        this.target = target;
    }

    /**
     * This method returns the current Ant project.
     *
     * @return the project of this AntTaskInfo
     */
    public Project getProject() {
        return project;
    }

    /**
     * This method returns the Ant task name.
     *
     * @return the ant task name of this AntTaskInfo
     */
    public String getTaskName() {
        return taskName;
    }

    /**
     * This method returns the Ant task location.
     *
     * @return the location of this AntTaskInfo
     */
    public Location getLocation() {
        return location;
    }

    /**
     * This method returns the Ant target being executed.
     *
     * @return the target of this AntTaskInfo
     */
    public Target getTarget() {
        return target;
    }
}
