/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import org.apache.tools.ant.DynamicElement;

import java.util.HashMap;
import java.util.Map;


/**
 * This class represents a "credentials" XML element to be used in the Ant build file. The credentials can include user
 * name, password and other information needed to authenticate to the server. A particular Server implementation
 * should require the specific credentials. This element represents some kind of a property map, each property is
 * described by one nested element. The name of nested element represents the property key, the "value" attribute -
 * property value.
 * <p/>
 * <p>Example of the element:<p>
 * <pre>
 * &lt;credentials&gt;
 *      &lt;user_name value="topcoder" /&gt;
 *      &lt;password value="some_password" /&gt;
 *      &lt;some_other_property="some_value"/&gt;
 * &lt;/credentials&gt;
 * </pre>
 * <p/>
 * This class is mutable, i.e. not thread-safe.
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class CredentialsElement implements DynamicElement {

    /**
     * Stores the property map represented by the "credentials" element. Keys are String instances, values are also
     * String instances. They are set with call to the {@link CredentialProperty#setValue()}.
     */
    private Map propertyMap = new HashMap();

    /**
     * Creates a CredentialsElement instance.
     */
    public CredentialsElement() {
        //this constructor is intentionally empty
    }

    /**
     * This method creates a nested element with a specified name. Nested elements are of type {@link
     * CredentialProperty}.
     *
     * @param name the name of nested element to create
     *
     * @return the created CredentialProperty instance
     *
     * @throws org.apache.tools.ant.BuildException
     *                                  when fails to create nested element
     * @throws IllegalArgumentException if name is <tt>null</tt> or empty (trim'd) string
     */
    public Object createDynamicElement(final String name) {
        if (name == null) {
            throw new IllegalArgumentException("The parameter named [name] was null.");
        }
        if (name.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [name] was an empty String.");
        }

        return new CredentialProperty(name, getPropertyMap());
    }

    /**
     * This method returns the property map represented by the <tt>credentials</tt> element.
     *
     * @return the property map represented by the "credentials" element, which is not a disconnected copy, as proper
     *         usage of the map is expected
     */
    public Map getPropertyMap() {
        return propertyMap;
    }
}
