/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import org.apache.tools.ant.BuildException;

import java.io.File;


/**
 * This class is a base class for all classes representing an actual server to be used while running tests. Derived
 * classes should have corresponding XML elements to be used in the Ant build file. <ul><li>Supported attributes:
 * <ul><li><tt>dir</tt> - required, defines the home directory where the server binaries are located, thus needed to
 * start up the server</li> <li><tt>port</tt> - non-required, specifies the port to be used when staring the server</li>
 * <li><tt>server</tt> - non-required, the hostname under which the started server shall be connected, defaults to
 * <tt>"localhost"</tt></li> <li><tt>startserver</tt> - non-required, specifies whether the server should be started
 * during test run preparation</li> <li><tt>initserver</tt> - non-required, specifies whether the server should be
 * initialized with test data during test run preparation</li> </ul></li><li>Nested elements: <ul><li>Can include single
 * <tt>credentials</tt> element, which is not required. </li></ul></li></ul>
 * <p/>
 * This class is mutable, i.e. not thread-safe.
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public abstract class Server {

    /**
     * This constant represents the key used in the credentials property map to pass the user name. It is used to
     * provide common interface for all derived classes. Derived classes should also define such constants for the keys
     * they need.
     */
    public static final String USER_NAME = "user_name";

    /**
     * This constant represents the key used in the credentials property map to pass the password. It is used to provide
     * common interface for all derived classes. Derived classes should also define such constants for the keys they
     * need.
     */
    public static final String PASSWORD = "password";
    /**
     * This field represents the <tt>dir</tt> attribute. This attribute represents the installation directory of the
     * server. This attribute is required. The field is initialized to <tt>null</tt> and set via the {@link
     * #setDir(java.io.File)} method.
     */
    private File dir = null;

    /**
     * This field represents the <tt>port</tt> attribute. This attribute represents the TCP port that the server should
     * listen to. This attribute is not required. The default value should be defined by the derived classes. The field
     * is initialized to <tt>-1</tt> and set via the {@link #setPort(int)} method.
     */
    private int port = -1;

    /**
     * This field represents the <tt>server</tt> attribute. This attribute represents the host name or IP address of the
     * machine running the server. This attribute is not required, default value is <tt>"localhost"</tt>. The field is
     * initialized to <tt>"localhost"</tt> and set via the {@link #setServer(String)} method.
     */
    private String server = "localhost";

    /**
     * This field represents the <tt>startserver</tt> attribute. This attribute specifies if the server should be
     * started by the Component while running tests. The field is initialized to <tt>true</tt> and set via the {@link
     * #setStartServer(boolean)} method.
     */
    private boolean startServer = true;

    /**
     * This field represents information about the current Ant project, task, etc. The field is initialized to
     * <tt>null</tt> and set via the {@link #setAntTaskInfo(AntTaskInfo)} method.
     */
    private AntTaskInfo antTaskInfo = null;

    /**
     * This field represents the <tt>initserver</tt> attribute. This attribute specifies if the server should be
     * initialized with data by the Component when running tests. The field is initialized to <tt>true</tt> and set via
     * the {@link #setInitServer(boolean)} method.
     */
    private boolean initServer = true;

    /**
     * Represents the nested <tt>credentials</tt> element, which contains the credentials used to authenticate to the
     * server. This can include user name, password, certificate, etc. The field is initialized to <tt>null</tt> and set
     * in the {@link #createCredentials()} method.
     */
    private CredentialsElement credentials = null;

    /**
     * Creates a Server instance. The type of server represented by the class is specified. Type is a string like
     * <tt>"tomcat5x"</tt>, which actually specifies the name and version of the server.
     *
     * @param type the type of the server
     *
     * @throws BuildException           if the type of the server is unrecognized, or any other error happens
     * @throws IllegalArgumentException if type is <tt>null</tt>
     */
    protected Server(final String type) {
        if (type == null) {
            throw new IllegalArgumentException("The parameter named [type] was null.");
        }
    }

    /**
     * This method starts the server. In the case when <tt>startserver</tt> attribute is set to <tt>false</tt>, the
     * server should not be started, but some initialization may still be needed. Subclasses must implement this method
     * to perform the actual task of starting up the server.
     *
     * @throws BuildException if any error happens
     */
    public abstract void startUp();

    /**
     * This method stops the server. The server should not be stopped if it wasn't started by the startUp() method, i.e.
     * <tt>startserver</tt> attribute is set to <tt>false</tt>. Subclasses must implement this method to perform the
     * actual task of stopping the server.
     *
     * @throws BuildException if any error happens
     */
    public abstract void shutDown();

    /**
     * This method initializes the server  with data. In the case when <tt>initserver</tt> attribute is set to
     * <tt>false</tt>, does nothing, or can check if the data are properly initialized, the implementation can choose
     * the preferred way. Subclasses must implement this method to perform the actual task of initializing server with
     * data.
     *
     * @throws BuildException if any error happens
     */
    public abstract void initData();

    /**
     * This method cleans up the data initialized by the {@link #initData()} method. In the case when
     * <tt>initserver</tt> attribute is set to <tt>false</tt> or data was not initialized, does nothing. Subclasses must
     * implement this method to perform the actual task of  cleaning-up the data.
     *
     * @throws BuildException if any error happens
     */
    public abstract void cleanUpData();

    /**
     * This method sets the value of the <tt>dir</tt> attribute, which represents the installation directory of the
     * server. This attribute is required.
     *
     * @param dir the value of the <tt>dir</tt> attribute to be set
     */
    public void setDir(final File dir) {
        this.dir = dir;
    }

    /**
     * This method returns the value of the <tt>dir</tt> attribute, which represents the installation directory of the
     * server.
     *
     * @return the value of the <tt>dir</tt> attribute
     */
    public File getDir() {
        return dir;
    }

    /**
     * This method sets the value of the <tt>port</tt> attribute, which represents the TCP port that the server should
     * listen to. This attribute is not required. The default value should be defined by the derived classes.
     *
     * @param port the value of the <tt>port</tt> attribute to be set
     *
     * @throws IllegalArgumentException if port is invalid (e.g. is non-positive)
     */
    public void setPort(final int port) {
        if (port < 1) {
            throw new IllegalArgumentException(
                "The parameter named [port] was expected to be >0 , but was [" + port + "].");
        }

        this.port = port;
    }

    /**
     * This method returns the value of the <tt>port</tt> attribute, which represents the TCP port that the server
     * should listen to.
     *
     * @return the value of the <tt>port</tt> attribute
     */
    public int getPort() {
        return port;
    }

    /**
     * This method sets the value of the <tt>server</tt> attribute, which represents the host name or IP address of the
     * machine running the server. This attribute is not required, default value is <tt>"localhost"</tt>.
     *
     * @param server the value of the <tt>server</tt> attribute to be set
     *
     * @throws IllegalArgumentException if server is empty string
     */
    public void setServer(final String server) {
        if (server != null && server.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [server] was an empty String.");
        }
        this.server = server;
    }

    /**
     * This method sets the value of the <tt>server</tt> attribute, which represents the host name or IP address of the
     * machine running the server.
     *
     * @return the value of the <tt>server</tt> attribute
     */
    public String getServer() {
        return server;
    }

    /**
     * This method sets the value of the <tt>startserver</tt> attribute, which specifies whether the server should be
     * started by the Component while running tests.
     *
     * @param startServer the value of the <tt>startserver</tt> attribute to be set
     */
    public void setStartServer(final boolean startServer) {
        this.startServer = startServer;
    }

    /**
     * This method returns the value of the <tt>startserver</tt> attribute, which specifies whether the server should be
     * started by the Component while running tests.
     *
     * @return the value of the <tt>startserver</tt> attribute
     */
    public boolean getStartServer() {
        return startServer;
    }

    /**
     * This method sets the AntTaskInfo which represents information about the current Ant project, task, etc.
     *
     * @param antTaskInfo the AntTaskInfo to be set
     *
     * @throws IllegalArgumentException if given antTaskInfo is <tt>null</tt>
     */
    public void setAntTaskInfo(final AntTaskInfo antTaskInfo) {
        if (antTaskInfo == null) {
            throw new IllegalArgumentException("The parameter named [antTaskInfo] was null.");
        }

        this.antTaskInfo = antTaskInfo;
    }

    /**
     * This method returns the AntTaskInfo which represents information about the current Ant project, task, etc.
     *
     * @return the AntTaskInfo describing the current environment
     */
    public AntTaskInfo getAntTaskInfo() {
        return antTaskInfo;
    }

    /**
     * This method sets the value of the <tt>initserver</tt> attribute, which specifies whether the server should be
     * initialized with data by the Component when running tests.
     *
     * @param initServer the value of the <tt>initserver</tt> attribute to be set
     */
    public void setInitServer(final boolean initServer) {
        this.initServer = initServer;
    }

    /**
     * This method returns the value of the <tt>initserver</tt> attribute, which specifies whether the server should be
     * initialized with data by the Component when running tests.
     *
     * @return the value of the <tt>initserver</tt> attribute
     */
    public boolean getInitServer() {
        return initServer;
    }

    /**
     * This method creates a nested <tt>credentials</tt> element.
     *
     * @return the created instance of {@link CredentialsElement}
     *
     * @throws BuildException if there are more than one nested elements (i.e. this method gets called more than once),
     *                        or when fails to create a nested element
     */
    public CredentialsElement createCredentials() {
        if (credentials != null) {
            throw new BuildException("This Server element already has a nested credentials element.");
        }
        credentials = new CredentialsElement();
        return credentials;
    }

    /**
     * This method returns the nested <tt>credentials</tt> element, which represents the credentials used to
     * authenticate to the server.
     *
     * @return the nested <tt>credentials</tt> element, or <tt>null</tt> if no nested credentials element exists
     */
    public CredentialsElement getCredentials() {
        return credentials;
    }
}
