/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DynamicElement;

import java.lang.reflect.Constructor;


/**
 * This class represents a <tt>server</tt> XML element to be used in the Ant build file. It acts as container for an XML
 * element representing the actual server to be used for running tests. The name of the element in the build file is not
 * actually <tt>server</tt>, but is the fully qualified class name of the {@link Server} subclass to be used as
 * implementation.
 * <p/>
 * <ul><li>Supported attributes: <ul><li>None</li></ul> </li><li>Nested elements: <ul><li>Should include one nested
 * element with a custom name, specifying server type. </li></ul></li></ul>
 * <p/>
 * This class is mutable, i.e. not thread-safe.
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class ServerElement implements DynamicElement {

    /**
     * This field represents the <tt>class</tt> attribute, which represents the name of class derived from {@link
     * Server}, which supports the server described by this element. This attribute is required.
     * <p/>
     * The field is initialized in the constructor.
     */
    private final String clazz;

    /**
     * This field represents the server described by this element.
     * <p/>
     * The field is initialized to <tt>null</tt> and set in the {@link #createDynamicElement(String)} method.
     */
    private Server server = null;

    /**
     * This field represents information about the current Ant project, task, etc. The field is initialized to
     * <tt>null</tt> and set via the {@link #setAntTaskInfo(AntTaskInfo)} method.
     */
    private AntTaskInfo antTaskInfo = null;

    /**
     * Creates a ServerElement instance.
     *
     * @param className the name of class derived from {@link Server}, which supports the server described by this
     *                  element
     *
     * @throws IllegalArgumentException incase the given className is <tt>null</tt> or an empty (trim'd) String
     */
    public ServerElement(final String className) {
        if (className == null) {
            throw new IllegalArgumentException("The parameter named [className] was null.");
        }
        if (className.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [className] was an empty String.");
        }
        clazz = className;
    }

    /**
     * This method creates an element with a specified name. Handles unknown elements. Really this methods creates an
     * instance of class derived from {@link Server} The name of the element specifies the type of the server to be
     * passed to class constructor. The name of actual class to create instance of is taken from <tt>class</tt>
     * attribute.
     *
     * @param name the name of element to be created, specifies the type of server this element describes
     *
     * @return an instance of class derived from AbstractServer, represents the actual server described by this element
     *
     * @throws BuildException           if can't create an instance of specified class via reflection, or if the created
     *                                  class doesn't support the specified server type
     * @throws IllegalArgumentException if name is <tt>null</tt> or empty (trim'd) string
     */
    public Object createDynamicElement(final String name) {
        if (name == null) {
            throw new IllegalArgumentException("The parameter named [name] was null.");
        }
        if (name.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [name] was an empty String.");
        }
        // Check if there was no previously created element
        if (server != null) {
            throw new BuildException("There should be only one nested element describing server.");
        }
        if (getAntTaskInfo() == null) {
            throw new BuildException("The dynamic element cannot be constructed as this ServerElement instance "
                + "does not have a valid AntTaskInfo instance assigned yet.");
        }

        //lookup and load instance implementation class
        final Class implClass;
        try {
            implClass = Class.forName(clazz);
        } catch (Throwable t) {
            // catch of Throwable is normally discouraged by the TC coding style, but
            // as a lot of Errors can arise from a class load and these
            // errors don't need to be let pass through this catch clause (as they
            // definitely belong to the class load and not the normal program
            // execution) all Throwables are caught here and
            // wrapped into a BuildException.
            throw new BuildException("Unable to load implementation class [" + clazz + "].", t);
        }

        //check if class matches expected type
        if (!Server.class.isAssignableFrom(implClass)) {
            throw new BuildException(
                "The implementation class [" + clazz + "]  is not of type [" + Server.class.getName() + "].");
        }

        //find constructor
        final Constructor constructor;
        try {
            constructor = implClass.getConstructor(new Class[]{String.class});
        } catch (NoSuchMethodException e) {
            throw new BuildException("The implementation class [" + implClass .getName()
                + "]  did not have a constructor with single String argument signature.", e);
        }

        //create instance
        try {
            server = (Server) constructor.newInstance(new Object[]{name});
        } catch (Throwable t) {
            // catch of Throwable is normally discouraged by the TC coding style, but
            // as a lot of Errors can arise from a reflection invocation and these
            // errors don't need to be let pass through this catch clause (as they
            // definitely belong to the reflective invocation and not the normal program
            // execution) all Throwables are caught here and
            // wrapped into a BuildException.
            throw new BuildException("the implementation class [" + clazz
                + "]  could not be instantiated, constructor invocation failed.", t);
        }
        server.setAntTaskInfo(getAntTaskInfo());
        return server;
    }

    /**
     * This method returns the server described by this element.
     *
     * @return the server described by this element
     */
    public Server getServer() {
        return server;
    }

    /**
     * This method sets the information about the current Ant project, task, etc.
     *
     * @param antTaskInfo the AntTaskInfo to be set
     *
     * @throws IllegalArgumentException if antTaskInfo is <tt>null</tt>
     */
    public void setAntTaskInfo(final AntTaskInfo antTaskInfo) {
        if (antTaskInfo == null) {
            throw new IllegalArgumentException("The parameter named [antTaskInfo] was null.");
        }

        this.antTaskInfo = antTaskInfo;
    }

    /**
     * This method returns the information about the current Ant project, task, etc.
     *
     * @return the ant Task Info
     */
    public AntTaskInfo getAntTaskInfo() {
        return antTaskInfo;
    }

}
