/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import org.apache.cactus.integration.ant.CactifyWarTask;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.ZipFileSet;

import java.io.File;


/**
 * This class represents an Ant task that injects elements necessary to run the tests into an existing WAR file. It can
 * be used to prepare WARs for {@link DefaultWebApplicationServer} class.
 * <p/>
 * This class is mutable, i.e. not thread-safe.
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class PrepareTestWarTask extends Task {

    /**
     * This field stores a wrapped {@link org.apache.cactus.integration.ant.CactifyWarTask} instance. It is initialized
     * in field initializer. The instance in this field is the delegate to which all implementation and functionality is
     * delegated.
     */
    private final CactifyWarTask cactifyWarTask = new CactifyWarTask();

    /**
     * Creates a new PrepareTestWarTask.
     */
    public PrepareTestWarTask() {
        // This constructor is intentionally empty
    }

    /**
     * This method sets the value of <tt>srcfile</tt> attribute, which specifies the source archive to inject the
     * elements required to run tests into.
     *
     * @param srcFile the source archive to inject the elements required to run tests into
     */
    public void setSrcFile(final File srcFile) {
        cactifyWarTask.setSrcFile(srcFile);
    }

    /**
     * This method sets the value of <tt>destfile</tt> attribute, which specifies the destination file, archive with the
     * injected elements required to run tests will be written to.
     *
     * @param destFile the destination file archive with the injected elements required to run tests will be written to
     */
    public void setDestFile(final File destFile) {
        cactifyWarTask.setDestFile(destFile);
    }

    /**
     * This method adds a fileset to the preparation task. The fileset is used to specify additional files that are to
     * be included in the prepared war file as content (i.e. JSP files to be put relative to the WAR root).
     *
     * @param set the fileset specifying the files to be added
     */
    public void addFileset(final FileSet set) {
        cactifyWarTask.addFileset(set);
    }

    /**
     * This method adds a ZipFileSet to the preparation task. The fileset is used to specify additional library jar
     * files that are to be included in the prepared war file as libraries (i.e. JAR files to be put relative to
     * <tt>/WEB-INF/lib</tt>).
     *
     * @param set the ZipFileSet specifying the files to be added
     */
    public void addLib(final ZipFileSet set) {
        cactifyWarTask.addLib(set);
    }

    /**
     * This method adds a ZipFileSet to the preparation task. The fileset is used to specify additional class files that
     * are to be included in the prepared war file as loadable classes (i.e. <tt>*.class</tt> files to be put relative
     * to <tt>/WEB-INF/classes</tt>).
     *
     * @param set the ZipFileSet specifying the files to be added
     */
    public void addClasses(final ZipFileSet set) {
        cactifyWarTask.addClasses(set);
    }

    /**
     * This method runs the preparation task.
     *
     * @throws org.apache.tools.ant.BuildException
     *          if any error happens
     */
    public void execute() {
        cactifyWarTask.setProject(getProject());
        try {
            cactifyWarTask.execute();
        } catch (Exception e) {
            // Catch of Exception is normally discouraged by TC style, but as we want
            // to 'shield' the caller from the internals of the wrapped implementation
            // of cactus (which sometimes can throw NPEs) we simply wrap everything
            // into BuildException to assure normal ant behavior
            throw new BuildException("Error while preparing the war file.", e);
        }
    }

}
