/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import org.apache.cactus.JspTestCase;

import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;


/**
 * This class represents JUnit test case to unit test code that needs an access to valid JSP implicit objects (such as
 * the page context, the output jsp writer, the HTTP request, ...) This implementation of the class is derived from
 * Cactus JspTestCase, and just adds more high level access to Jsp objects. Jsp objects should be accessed by the
 * derived classes using the getter methods provided by this class. Derived classes should not use directly methods and
 * fields defined by the Cactus framework.
 * <p/>
 * This class is mutable, i.e. not thread-safe.
 * <p/>
 * For each XXX test case, you can define a corresponding beginXXX() method(optional). It will be run on the client
 * side. It can be used to initialize HTTP related parameters (HTTP parameters, cookies, HTTP headers, URL to simulate,
 * ...). One will be able to retrieve these values in your testXXX() by calling the different API of HttpServletRequest
 * (like getQueryString(), getCookies(), getHeader(), ...).
 * <p/>
 * The signature of the begin method is:
 * <p/>
 * <pre>
 * public void beginXXX(org.apache.cactus.WebRequest theRequest) {
 *   [...]
 * }
 * </pre>
 * <p/>
 * where theRequest is the object  that you use to set all the HTTP related parameters.
 * <p/>
 * For each XXX test case, you can define a corresponding endXXX() method. This method is called on the client side. You
 * will use this method to verify the returned HTTP related parameters from your test case (like the returned content of
 * the HTTP response, any returned cookies, returned HTTP headers, ...).
 * <p/>
 * The signature of the end method is:
 * <pre>
 * public void endXXX(org.apache.cactus.WebResponse theResponse) {
 *   [...]
 * }
 * </pre>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class TCJspTestCase extends JspTestCase {

    /**
     * Default constructor defined in order to allow creating Test Case without needing to define constructor (new
     * feature in JUnit 3.8.1).
     */
    public TCJspTestCase() {
        super();
    }

    /**
     * Constructs a TCJspTestCase with the given name.
     *
     * @param name the name of the test case
     */
    public TCJspTestCase(final String name) {
        super(name);
    }

    /**
     * This method returns a valid {@link javax.servlet.ServletConfig} object that can be accessed from the
     * <tt>testXXX()</tt>, {@link #setUp()} and {@link #tearDown()} methods.
     * <p/>
     * <b>Note:</b> Calling this method from either the <tt>beginXXX()</tt> or <tt>endXXX()</tt> methods will return
     * <tt>null</tt>.
     *
     * @return the {@link javax.servlet.ServletConfig} object currently in scope
     */
    public ServletConfig getConfig() {
        return config;
    }

    /**
     * This method returns a valid {@link HttpServletRequest} object that can be accessed from the <tt>testXXX()</tt>,
     * {@link #setUp()} and {@link #tearDown()} methods.
     * <p/>
     * <b>Note:</b> Calling this method from either the <tt>beginXXX()</tt> or <tt>endXXX()</tt> methods will return
     * <tt>null</tt>.
     *
     * @return the {@link HttpServletRequest} object currently in scope
     */
    public HttpServletRequest getRequest() {
        return request;
    }

    /**
     * This method returns a valid {@link HttpServletResponse} object that can be accessed from the <tt>testXXX()</tt>,
     * {@link #setUp()} and {@link #tearDown()} methods.
     * <p/>
     * <b>Note:</b> Calling this method from either the <tt>beginXXX()</tt> or <tt>endXXX()</tt> methods will return
     * <tt>null</tt>.
     *
     * @return the {@link HttpServletResponse} object currently in scope
     */
    public HttpServletResponse getResponse() {
        return response;
    }

    /**
     * This method returns a valid {@link HttpSession} object that can be accessed from the <tt>testXXX()</tt>, {@link
     * #setUp()} and {@link #tearDown()} methods.
     * <p/>
     * <b>Note:</b> Calling this method from either the <tt>beginXXX()</tt> or <tt>endXXX()</tt> methods will return
     * <tt>null</tt>.
     *
     * @return the {@link HttpSession} object currently in scope
     */
    public HttpSession getSession() {
        return session;
    }

    /**
     * This method returns a valid {@link JspWriter} object that can be accessed from the <tt>testXXX()</tt>, {@link
     * #setUp()} and {@link #tearDown()} methods.
     * <p/>
     * <b>Note:</b> Calling this method from either the <tt>beginXXX()</tt> or <tt>endXXX()</tt> methods will return
     * <tt>null</tt>.
     *
     * @return the {@link JspWriter} object currently in scope
     */
    public JspWriter getOut() {
        return out;
    }

    /**
     * This method returns a valid {@link PageContext} object that can be accessed from the <tt>testXXX()</tt>, {@link
     * #setUp()} and {@link #tearDown()} methods.
     * <p/>
     * <b>Note:</b> Calling this method from either the <tt>beginXXX()</tt> or <tt>endXXX()</tt> methods will return
     * <tt>null</tt>.
     *
     * @return the {@link PageContext} object currently in scope
     */
    public PageContext getPageContext() {
        return pageContext;
    }
}
