/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import org.apache.cactus.ServletTestCase;

import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;


/**
 * This class represents JUnit test case to unit test code that needs an access to valid Servlet implicit objects (such
 * as the HTTP request, the HTTP response, the servlet config, ...) This implementation of the class is derived from
 * Cactus ServletTestCase, and just adds more high level access to Servlet objects. Servlet objects should be accessed
 * by the derived classes using the getter methods provided by this class. Derived classes should not use directly
 * methods and fields defined by the Cactus framework.
 * <p/>
 * This class is mutable, i.e. not thread-safe.
 * <p/>
 * For each XXX test case, you can define a corresponding beginXXX() method(optional). It will be run on the client
 * side. It can be used to initialize HTTP related parameters (HTTP parameters, cookies, HTTP headers, URL to simulate,
 * ...). One will be able to retrieve these values in your testXXX() by calling the different API of HttpServletRequest
 * (like getQueryString(), getCookies(), getHeader(), ...).
 * <p/>
 * The signature of the begin method is:
 * <p/>
 * <pre>
 * public void beginXXX(org.apache.cactus.WebRequest theRequest) {
 *   [...]
 * }
 * </pre>
 * <p/>
 * where theRequest is the object  that you use to set all the HTTP related parameters.
 * <p/>
 * For each XXX test case, you can define a corresponding endXXX() method. This method is called on the client side. You
 * will use this method to verify the returned HTTP related parameters from your test case (like the returned content of
 * the HTTP response, any returned cookies, returned HTTP headers, ...).
 * <p/>
 * The signature of the end method is:
 * <pre>
 * public void endXXX(org.apache.cactus.WebResponse theResponse) {
 *   [...]
 * }
 * </pre>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class TCServletTestCase extends ServletTestCase {

    /**
     * This is a Default constructor defined in order to allow creating Test Case without needing to define constructor
     * (new feature in JUnit 3.8.1).
     */
    public TCServletTestCase() {
        super();
    }

    /**
     * Constructs a TCServletTestCase test case with the given name.
     *
     * @param name the name of the test case
     */
    public TCServletTestCase(final String name) {
        super(name);
    }

    /**
     * This method returns a valid {@link javax.servlet.ServletConfig} object that can be accessed from the
     * <tt>testXXX()</tt>, {@link #setUp()} and {@link #tearDown()} methods.
     * <p/>
     * <b>Note:</b> Calling this method from either the <tt>beginXXX()</tt> or <tt>endXXX()</tt> methods will return
     * <tt>null</tt>.
     *
     * @return the {@link javax.servlet.ServletConfig} object currently in scope
     */
    public ServletConfig getConfig() {
        return config;
    }

    /**
     * This method returns a valid {@link HttpServletRequest} object that can be accessed from the <tt>testXXX()</tt>,
     * {@link #setUp()} and {@link #tearDown()} methods.
     * <p/>
     * <b>Note:</b> Calling this method from either the <tt>beginXXX()</tt> or <tt>endXXX()</tt> methods will return
     * <tt>null</tt>.
     *
     * @return the {@link HttpServletRequest} object currently in scope
     */
    public HttpServletRequest getRequest() {
        return request;
    }

    /**
     * This method returns a valid {@link HttpServletResponse} object that can be accessed from the <tt>testXXX()</tt>,
     * {@link #setUp()} and {@link #tearDown()} methods.
     * <p/>
     * <b>Note:</b> Calling this method from either the <tt>beginXXX()</tt> or <tt>endXXX()</tt> methods will return
     * <tt>null</tt>.
     *
     * @return the {@link HttpServletResponse} object currently in scope
     */
    public HttpServletResponse getResponse() {
        return response;
    }

    /**
     * This method returns a valid {@link HttpSession} object that can be accessed from the <tt>testXXX()</tt>, {@link
     * #setUp()} and {@link #tearDown()} methods.
     * <p/>
     * <b>Note:</b> Calling this method from either the <tt>beginXXX()</tt> or <tt>endXXX()</tt> methods will return
     * <tt>null</tt>.
     *
     * @return the {@link HttpSession} object currently in scope
     */
    public HttpSession getSession() {
        return session;
    }
}
