/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import com.topcoder.testframework.AbstractApplicationServerTest;
import com.topcoder.testframework.AntTaskInfo;
import com.topcoder.testframework.ApplicationServer;
import com.topcoder.testframework.TestingUtils;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;

import java.io.File;
import java.net.URL;


/**
 * This class contains the unit tests for {@link com.topcoder.testframework.Server}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class DefaultWebApplicationServerTest extends AbstractApplicationServerTest {
    /**
     * This is the instance to be tested, created in {@link #setUp()}.
     */
    private DefaultWebApplicationServer webApplicationServer;
    /**
     * This is an AntTaskInfo instance used in tests.
     */
    private AntTaskInfo antTaskInfo;

    /**
     * This method tests {@link DefaultWebApplicationServer#startUp()} and {@link DefaultWebApplicationServer
     * #shutDown()} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testStartUpShutdown() throws Exception {
        webApplicationServer.setWarFile(new File(TestingUtils.VALID_PREPARED_WAR_FILE));
        webApplicationServer.setAntTaskInfo(antTaskInfo);
        webApplicationServer.setDir(new File(TestingUtils.getValidTomcatHomeDir()));
        webApplicationServer.setPort(TestingUtils.getPort());

        webApplicationServer.startUp();

        webApplicationServer.shutDown();
    }

    /**
     * This method tests the correct failure behavior of {@link DefaultWebApplicationServer#startUp()}.
     * <p/>
     * <b>Failure reason</b>: both ear and war file are specified
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testStartupFailEarAndWarSpecified() throws Exception {
        webApplicationServer.setWarFile(new File(TestingUtils.VALID_PREPARED_WAR_FILE));
        webApplicationServer.setEarFile(new File(TestingUtils.VALID_UNPREPARED_EAR_FILE));
        webApplicationServer.setAntTaskInfo(
            antTaskInfo);
        webApplicationServer.setDir(new File(TestingUtils.getValidTomcatHomeDir()));
        webApplicationServer.setPort(TestingUtils.getPort());
        try {
            webApplicationServer.startUp();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DefaultWebApplicationServer#startUp()}.
     * <p/>
     * <b>Failure reason</b>: neither ear nor war file are specified and no testURL specified
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testStartupFailNeitherEarNorWarSpecified() throws Exception {
        webApplicationServer.setAntTaskInfo(
            antTaskInfo);
        webApplicationServer.setDir(new File(TestingUtils.getValidTomcatHomeDir()));
        webApplicationServer.setPort(TestingUtils.getPort());
        try {
            webApplicationServer.startUp();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link DefaultWebApplicationServer#setTimeout(long)}  and {@link
     * DefaultWebApplicationServer#getTimeout()}for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetSetTimeout() throws Exception {
        assertEquals("unexpected initial value", 180000, webApplicationServer.getTimeout());
        final int timeout = 2000;
        webApplicationServer.setTimeout(timeout);
        assertEquals("setter does not work", timeout, webApplicationServer.getTimeout());
    }

    /**
     * This method tests the correct failure behavior of {@link DefaultWebApplicationServer#setTimeout(long)}.
     * <p/>
     * <b>Failure reason</b>: negative timeout given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testSetTimeoutFailNegativeTimeout() throws Exception {
        try {
            webApplicationServer.setTimeout(-3);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link DefaultWebApplicationServer#setTestURL(java.net.URL)}  and {@link
     * DefaultWebApplicationServer#getTestURL()}for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetSetTestUrl() throws Exception {
        assertEquals("unexpected initial value", null, webApplicationServer.getTestURL());
        final URL testUrl = new URL("http://test");
        webApplicationServer.setTestURL(testUrl);
        assertEquals("setter does not work", testUrl, webApplicationServer.getTestURL());
    }

    /**
     * This method creates the {@link com.topcoder.testframework.ApplicationServer} instance to be tested by calling the
     * constructor using the given type value as argument.
     *
     * @param type the type value to be used as constructor argument
     *
     * @return the instance created
     */
    protected ApplicationServer createApplicationServer(final String type) {
        return new DefaultWebApplicationServer(type);
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        webApplicationServer = new DefaultWebApplicationServer(TYPE);
        final Project project = new Project();
        project.init();
        antTaskInfo = new AntTaskInfo(project, "task", Location.UNKNOWN_LOCATION, new Target());
    }

    /**
     * This method does the cleanup after test run.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        try {
            webApplicationServer.shutDown();
        } catch (Exception e) {
            // ignored as we want to assure the
            // container is shut down, but don't know
            // for sure whether it has been started up,
            // so we try to shut down but don't bother
            // when shutdown fails
        }
    }
}
